/*
   Project: PcapKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-23 09:47:49 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "PcapKit.h"

// callback function used by pcap_loop
void
got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet);

/*
 * callback function called from pcap_loop
 */
void
got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
{
  NSData *pData = [NSData dataWithBytes:packet length:header->len];
  if ([[(id)args delegate] conformsToProtocol:@protocol(PcapKitDelegate)])
    {
      [(id<PcapKitDelegate>)[(id)args delegate] handlePacket:pData];
    }
}

@implementation PcapKit
- (id) init
{
  if ((self = [super init]) != nil)
    {
      device = NULL;
      inputFile = NULL;
      outputFile = NULL;
      handle = NULL;
      readTimeout = 0;
      snaplen = BUFSIZ;
      bpfFilter = NULL;
    }
  return self;
}

- (void) dealloc
{
  [device release];
  [inputFile release];
  [outputFile release];
  [bpfFilter release];
  [delegate release];
  pcap_close(handle);
  free(errBuf);
  [super dealloc];
}

// class methods

+(NSString *) getDataLinkNameForType:(NSInteger) _type
{
  return [NSString stringWithFormat: @"%s", pcap_datalink_val_to_name(_type)];
}
+(NSInteger) getDataLinkTypeForName:(NSString *) _name
{
  return pcap_datalink_name_to_val([_name UTF8String]);
}
+(NSString *) getDataLinkDescriptionForType:(NSInteger)_type
{
  return [NSString stringWithFormat: @"%s",
		pcap_datalink_val_to_description(_type)];
}
+(NSString *) getErrorStringForStatus:(NSInteger *)_status
{
  return [NSString stringWithFormat:@"%s", pcap_statustostr((int)_status)];
}
+(NSString *) getErrorStringForType:(NSInteger)_type
{
  return [NSString stringWithFormat:@"%s", pcap_strerror((int) _type)];
}
+(NSString *) getPcapLibraryVersion
{
  return [NSString stringWithFormat:@"%s", pcap_lib_version()];
}

- (pcap_t *) handle
{
  return handle;
}

- (void) setDevice: (NSString *)_dev
{
  ASSIGN(device,_dev);
}
- (NSString *) device
{
  return device;
}

-(void) setInputFile: (NSString *) _if
{
  ASSIGN(inputFile,_if);
}
-(NSString *) inputFile
{
  return inputFile;
}

-(void) setOutputFile: (NSString *) _of
{
  ASSIGN(outputFile,_of);
}
-(NSString *) outputFile
{
  return outputFile;
}

- (void) setDirection: (pcap_direction_t) _direction
{
  direction = _direction;
  if (handle != NULL)
    pcap_setdirection(handle, _direction);
}
- (pcap_direction_t) direction
{
  return direction;
}

- (void) setDataLink: (NSInteger)_dlt
{
  pcap_set_datalink(handle, (int)_dlt);
}

- (void) setPromisc: (BOOL)_promisc
{
  promisc = _promisc;
}
- (BOOL) promisc
{
  return promisc;
}

-(void) setDelegate: (id) _delegate
{
  ASSIGN(delegate,_delegate);
}
-(id) delegate
{
  return delegate;
}

-(void) setReadTimeout:(NSInteger) _rt
{
  readTimeout = _rt;
  if (handle != NULL)
    pcap_set_timeout(handle, _rt);
}
-(NSInteger) readTimeout
{
  return readTimeout;
}

-(void) setBpfFilter: (NSString *) _bpf
{
  ASSIGN(bpfFilter,_bpf);
}
-(NSString *) bpfFilter
{
  return bpfFilter;
}

- (void) setOptimizeFilter: (BOOL) _optimize
{
  optimizeFilter = _optimize;
}
- (BOOL) optimizeFilter
{
  return optimizeFilter;
}

- (void) setBufsize:(NSInteger)_bufsize
{
  bufsize = (NSInteger) _bufsize;
  if (handle != NULL)
    pcap_set_buffer_size(handle, (int) _bufsize);
}
- (NSInteger) bufsize
{
  return (NSInteger) bufsize;
}

-(void) openLiveStream
{
  if (device == NULL)
    {
      NSLog(@"no device given");
      return;
    }

  handle = pcap_open_live([device UTF8String],
		(int)snaplen, (int)promisc, (int)readTimeout, errBuf);
  if (handle == NULL)
    {
      NSLog(@"Couldn't open device %@: %s", device, errBuf);
      abort();
    }
}
-(void) openOfflineStream
{
  handle = pcap_open_offline([inputFile UTF8String], errBuf);
  if (handle == NULL)
    {
      NSLog(@"Couldn't open file for reading: %@: %s", inputFile, errBuf);
      abort();
    }
}

-(BOOL)applyFilter
{
  // check for a valid pcap handle
  if (handle == NULL)
    return NO;

  // first compile the filter
  [self lookupNetmask];
  if (pcap_compile(handle, &filterProgram, 
      [bpfFilter UTF8String], 0, filterNetmask) == -1)
    {
      NSLog(@"Couldn't parse filter %@: %@",
			bpfFilter, [self getErrorString]);
      return NO;
    }
  if (pcap_setfilter(handle, &filterProgram) == -1)
    {
      NSLog(@"Couldn't install filter %@: %@", 
			bpfFilter, [self getErrorString]);
      return NO;
    }
  // free the filter program, not needed anymore  
  pcap_freecode(&filterProgram);
  return YES;
}

-(NSInteger) startPcapLoop
{
  // start the loop on the handle, until error occurs
  // call got_packet when packet was read
  return pcap_loop(handle, -1, got_packet, (u_char *) self);
}

-(void) stopPcapLoop
{
  pcap_breakloop(handle);
}

-(NSInteger) getDatalinksIntoArray:(int **)_dlts
{
  if (handle == NULL)
    return -1;

  return pcap_list_datalinks(handle, _dlts);
}

-(BOOL)lookupDevice
{
  char *dev;
  dev = pcap_lookupdev(errBuf);
  if (dev == NULL)
    {
      NSLog(@"unable to find a suitable device for sniffing: %s", errBuf);
      return NO;
    }
  else
    {
      [self setDevice:[NSString stringWithFormat:@"%s", dev]];
      NSLog(@"found device: %@", device);
      return YES;
    }
}

-(NSInteger)lookupNetmask
{
  NSInteger result;
  if (!device)
    {
      filterNetmask = 0;
      IP = 0;
      return -1;
    }
  result = pcap_lookupnet([device UTF8String],
                        &IP, &filterNetmask, errBuf);
  if (result == -1)
    {
      NSLog(@"Can't get netmask for device %@: %s", device, errBuf);
      filterNetmask = 0;
      IP = 0;
    }
  return result;
}

// figure out the data link of current sniffer
-(NSInteger) dataLink
{
  return pcap_datalink(handle);
}

-(NSString *)getErrorString
{
  if (handle == NULL)
    return nil;

  return [NSString stringWithFormat:@"%s", pcap_geterr(handle)];
}

-(void) printErrorWithPrefix:(NSString *)_prefix
{
  if (handle != NULL)
    pcap_perror(handle, [_prefix UTF8String]);
}

-(NSInteger)getSnaplen
{
  return pcap_snapshot(handle);
}

-(BOOL)isSavefileSwapped
{
  if (handle != NULL)
    if (pcap_is_swapped(handle) > 0)
      return YES;
  return NO;
}

-(NSInteger) getMajorVersion
{
  if (handle != NULL)
    return (NSInteger) pcap_major_version(handle);
  return -1;
}

-(NSInteger) getMinorVersion
{
  if (handle != NULL)
    return (NSInteger) pcap_minor_version(handle);
  return -1;
}

-(BOOL)checkSetRFMon
{
  if (pcap_can_set_rfmon(handle))
    return YES;
  else
    return NO;
}

-(BOOL) createSniffingHandle
{
  if (device != NULL)
    { 
      handle = pcap_create([device UTF8String], errBuf);
      return YES;
    }
  else if (inputFile != NULL)
    {
      handle = pcap_create([inputFile UTF8String], errBuf);
      return YES;
    }
  return NO;
}

-(NSInteger) activateSniffingHandle
{
  if (handle != NULL)
    return (NSInteger) pcap_activate(handle);
  return -1;
}

-(BOOL)enableRFMon:(BOOL)_flag
{
  if (handle == NULL)
    return NO;

  if(pcap_set_rfmon(handle,_flag))
    return YES;
  else
    return NO;
}

-(BOOL)enablePromiscuousMode:(BOOL)_flag
{
  promisc = _flag;
  if (handle == NULL)
    return NO;
  if (pcap_set_promisc(handle, _flag))
    return YES;
  else
    return NO;
}

-(BOOL) setSnaplen:(NSInteger)_snaplen
{
  snaplen = _snaplen;
  if (handle == NO)
    return NO;
  if (pcap_set_snaplen(handle,(int)_snaplen))
    return YES;
  else
    return NO;
}

@end
